//methods.dart
//noi luu tru cac methods, function dung chung

import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:wowit/data.dart';
import 'package:wowit/message.dart';
import 'package:wowit/url.dart';

class Methods {}

double round(double d, int r) {
  String s = d.toStringAsFixed(r);
  return double.parse(s);
}

String showDate(DateTime date) {
  return '${date.day}/${date.month}/${date.year}';
}

//do khoan cach thoi gian bien truyen vao so voi hien tai
String timeAgoString(DateTime date) {
  var secondsInterval = DateTime.now().difference(date).inSeconds;
  if (secondsInterval < 10) {
    return getMessage('VAI_GIAY_TRUOC');
  }
  if (secondsInterval < 60) {
    return secondsInterval.toString() + ' ' + getMessage('GIAY_TRUOC');
  }
  double minutes = (secondsInterval / 60.0);
  minutes = round(minutes, 1);
  if (minutes < 60) {
    return minutes.toString() + ' ' + getMessage('PHUT_TRUOC');
  }
  var hours = round((minutes / 60.0), 1);
  if (hours < 24) {
    return hours.toString() + ' ' + getMessage('GIO_TRUOC');
  }
  var days = round((hours / 60.0), 1);
  if (days < 30) {
    return days.toString() + ' ' + getMessage('NGAY_TRUOC');
  }
  var months = round((days / 30.0), 1);
  if (months < 12) {
    return months.toString() + ' ' + getMessage('THANG_TRUOC');
  }
  var years = round((months / 12.0), 1);
  return years.toString() + ' ' + getMessage('NAM_TRUOC');
}

dynamic getDataHashMap(Map data, String key, {String defaultValue = ""}) {
  try {
    dynamic val;
    if (data.containsKey(key)) {
      val = data[key];
      if (val != null && val.toString() != "") {
        return val;
      }
    }
    return defaultValue;
  } catch (e) {
    print('getDataHashMap ex:' + e.toString());
    return defaultValue;
  }
}

dynamic getDataHashMapList(Map data, String key, int index,
    {String defaultValue = ""}) {
  try {
    var arrKeys = key.split('.');
    if (arrKeys.length != 2) return defaultValue;
    if (data.containsKey(arrKeys[0])) {
      List<dynamic> lstObj = data[arrKeys[0]] as List<dynamic>;
      if (lstObj != null && lstObj.length > index) {
        Map map = lstObj[index];
        return getDataHashMap(map, arrKeys[1], defaultValue: defaultValue);
      }
    }
    return defaultValue;
  } catch (e) {
    print('getDataHashMap ex:' + e.toString());
    return defaultValue;
  }
}

String getFullURLImage(String collection, int documentID, String fileName) {
  var url = getURL('HOST') +
      '/' +
      getURL('HOST_PHOTO_SUB') +
      '/' +
      getURL('HOST_PHOTO_FOLDER') +
      '/' +
      collection +
      '/' +
      documentID.toString() +
      '/' +
      fileName;
  return url;
}

double stringToDouble(String s) {
  try {
    double d = double.parse(s);
    return d;
  } catch (e) {
    print('stringToDouble:' + e.toString());
    return 0;
  }
}

int stringToInt(String s) {
  try {
    double d = stringToDouble(s);
    return d.floor();
    // int i = int.parse(strs[0]);
    // return i;
  } catch (e) {
    print('stringToInt:' + e.toString());
    return 0;
  }
}

bool stringToBool(String s, {bool defaultValue = false}) {
  try {
    return s == "1" || s.toLowerCase() == "true";
  } catch (e) {
    print('stringToBool:' + e.toString());
    return defaultValue;
  }
}

DateTime stringToDateTime(String s) {
  try {
    var parsedDate = DateTime.parse(s);
    return parsedDate;
  } catch (e) {
    print('stringToTimestamp:' + e.toString());
    return new DateTime(1990);
  }
}

//doc tien thanh chu
//lienHe : neu so tien = 0 -> tra ve 'Lien He'
String docTienThanhChu(double soTien,
    {int roundDecimal = 2, bool lienHe = true}) {
  int i;

  if (soTien > 8999999999999999) {
    return soTien.toString();
  }

  List<String> tienDonVi = [
    getMessage('DON_VI_TIEN_TRIEU_TY'),
    getMessage('DON_VI_TIEN_NGIN_TY'),
    getMessage('DON_VI_TIEN_TY'),
    getMessage('DON_VI_TIEN_TRIEU'),
    getMessage('DON_VI_TIEN_NGIN'),
    getMessage('DON_VI_TIEN_DONG')
  ];

  List<double> tienSo = [
    1000000000000000.0,
    1000000000000.0,
    1000000000.0,
    1000000.0,
    1000.0,
    1
  ];
  for (i = 0; i < tienSo.length; i++) {
    if (soTien / tienSo[i] > double.maxFinite ||
        soTien / tienSo[i] < double.minPositive) {
      return soTien.toString();
    }

    if (soTien > tienSo[i]) {
      if (soTien % tienSo[i] > 0) {
        if (roundDecimal > 0) {
          var r = round((soTien / tienSo[i]), roundDecimal);

          return r.toString() + " " + tienDonVi[i];
        } else {
          double lastValue = (soTien / tienSo[i]);

          if (lastValue == 0 && lienHe) {
            return getMessage('GIA_LIEN_HE');
          }
          return (lastValue).toString() + " " + (tienDonVi[i]);
        }
      } else {
        return (stringToInt((soTien / tienSo[i]).toString())).toString() +
            " " +
            tienDonVi[i];
      }
    }
  }
  if (soTien == 0 && lienHe) {
    return getMessage('GIA_LIEN_HE');
  }
  return soTien.toString();
}

String getChanelId(int uidFrom, int uidTo) {
  return (uidFrom > uidTo)
      ? (uidFrom.toString() + '_' + uidTo.toString())
      : (uidTo.toString() + '_' + uidFrom.toString());
}

List<String> getDataHashMapListString(Map data, String key,
    {String defaultValue = ""}) {
  List<String> lst = [];
  try {
    var arrKeys = key.split('.');
    if (arrKeys.length != 2) return lst;
    if (data.containsKey(arrKeys[0])) {
      List<dynamic> lstObj = data[arrKeys[0]] as List<dynamic>;
      if (lstObj != null && lstObj.length > 0) {
        for (int index = 0; index < lstObj.length; index++) {
          Map map = lstObj[index];
          lst.add(getDataHashMap(map, arrKeys[1], defaultValue: defaultValue));
        }
      }
      return lst;
    }
    return lst;
  } catch (e) {
    return lst;
  }
}

List<Map> getDataHashMapListObject(Map data, String key) {
  List<Map> lst = [];
  try {
    if (data.containsKey(key)) {
      List<dynamic> lstObj = data[key] as List<dynamic>;
      if (lstObj != null && lstObj.length > 0) {
        for (int index = 0; index < lstObj.length; index++) {
          Map map = lstObj[index];
          lst.add(map);
        }
      }
      return lst;
    }
    return lst;
  } catch (e) {
    print('getDataHashMapListObject ex:' + e.toString());
    return lst;
  }
}

CollectionReference getCollection(String collectionName) {
  return FirebaseFirestore.instance
      .collection(mainCollection)
      .doc(mainDocument)
      .collection(collectionName);
}

Query getQuery(String collectionName,
    {Map<String, Object> conditions,
    Map<String, bool> orderBy,
    int limit = -1}) {
  if (limit == null) {
    limit = -1;
  }

  Query collection = FirebaseFirestore.instance
      .collection(mainCollection)
      .doc(mainDocument)
      .collection(collectionName);
  try {
    if (conditions != null && conditions.length > 0) {
      conditions.forEach((key, value) {
        collection = collection.where(key, isEqualTo: value);
      });
    }

    if (orderBy != null && orderBy.length > 0) {
      orderBy.forEach((key, value) {
        collection = collection.orderBy(key, descending: value);
      });
    }
    if (limit > 0) {
      collection = collection.limit(limit);
    }
  } catch (e) {
    print('getCollection : ' + e.toString());
  }

  return collection;
}

int getNewDocumentID() {
  String documentId =
      DateTime.now().difference(DateTime.utc(2000, 1, 1)).inSeconds.toString();
  return stringToInt(documentId);
}

DateTime stringToDatime(String dateTime) {
  return DateTime.parse(dateTime);
}

String preparePhone(String phone) {
  String phoneNumber;

  if (phone.startsWith('0')) {
    phoneNumber = "+84" + phone.substring(1);
  } else {
    phoneNumber = phone;
  }
  return phoneNumber;
}

void showWatting() {
  showDialog(
    context: Get.overlayContext,
    barrierDismissible: false,
    builder: (_) => WillPopScope(
      onWillPop: () async => false,
      child: Center(
        child: SizedBox(
          width: 60,
          height: 60,
          child: CircularProgressIndicator(
            strokeWidth: 10,
          ),
        ),
      ),
    ),
  );
}
